<?php
/**
 * Copyright © 2015 FiloBlu . All rights reserved.
 */

namespace FiloBlu\Flow\Helper;

use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\Helper\AbstractHelper;
use Magento\Framework\App\Helper\Context;
use Magento\Framework\App\ObjectManager;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\DB\Adapter\AdapterInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\StoreManagerInterface;

/**
 * Class Data
 * @package FiloBlu\Flow\Helper
 */
class Data extends AbstractHelper
{
    /**
     * @var string
     */
    const CONFIG_SPECIAL_ACTIONS_PRIORITIES = 'filoblu_flow/flow_special_actions/special_actions_priorities';

    /**
     * @var AdapterInterface
     */
    protected $_connection;

    /**
     * @var
     */
    protected $_storesByLocale;

    /**
     * @var ScopeConfigInterface
     */
    protected $scopeConfig;

    /**
     * @var StoreManagerInterface
     */
    protected $storeManager;

    /**
     * Data constructor.
     * @param Context $context
     * @param ResourceConnection $_connection
     * @param StoreManagerInterface $storeManager
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Context $context,
        ResourceConnection $_connection,
        StoreManagerInterface $storeManager,
        ScopeConfigInterface $scopeConfig
    ) {
        $this->_connection = $_connection->getConnection();
        $this->storeManager = $storeManager;
        $this->scopeConfig = $scopeConfig;
        parent::__construct($context);
    }

    /**
     * @return mixed
     */
    public function getModels()
    {
        $channels_in = (Array)$this->scopeConfig->getValue('processor/channels/in');
        $channels_out = (Array)$this->scopeConfig->getValue('processor/channels/out');

        $classes['select'] = '';

        foreach ($channels_in as $key => $channel) {
            $classes[$channel] = $key;
        }

        foreach ($channels_out as $key => $channel) {
            $classes[$channel] = $key;
        }

        return $classes;
    }

    /**
     * @param $channel_input
     * @return int|string|void
     */
    public function getChannelModels($channel_input)
    {
        $channels_in = $this->scopeConfig->getValue('processor/channels/in');

        foreach ($channels_in as $key => $channel) {
            if ($channel === $channel_input) {
                return $key;
            }
        }
    }

    /**
     * @param $idInboundflow
     * @return array
     */
    public function getChannelByInbound($idInboundflow)
    {
        if ($idInboundflow > 0) {
            $result = $this->_connection->fetchRow(
                'SELECT * FROM flow_inboundflow WHERE id = ? LIMIT 1',
                [$idInboundflow]
            );

            if (count($result) > 0) {
                return $result;
            }
        }

        return [];
    }

    /**
     * @param $filepath
     * @return array
     */
    public function file_get_php_classes($filepath)
    {
        $php_code = file_get_contents($filepath);
        $classes = $this->get_php_classes($php_code);
        return $classes;
    }

    /**
     * @param $php_code
     * @return array
     */
    public function get_php_classes($php_code)
    {
        $tokens = token_get_all($php_code);
        $count = count($tokens);
        for ($i = 2; $i < $count; $i++) {
            if ($tokens[$i - 2][0] === T_CLASS
                && $tokens[$i - 1][0] === T_WHITESPACE
                && $tokens[$i][0] === T_STRING) {
                $class_name = $tokens[$i][1];
                $pieces = explode('_', $class_name);
                $key = $end = end($pieces);
                if ($key !== 'Abstract') {
                    return $class_name;
                }
            }
        }
        return [];
    }

    /**
     * @param $text
     * @return string
     */
    public function removeTextSpaces($text)
    {
        $result = preg_replace("/\s/u", '', $text);
        $result = preg_replace('/( )+/u', '', $result);
        return trim($result);
    }

    /**
     * @return array
     */
    public function getAllStoreIds()
    {
        $storeIds = [];

        foreach ($this->storeManager->getStores($withDefault = true) as $store) {
            $storeIds[$store->getCode()] = $store->getId();
        }

        return $storeIds;
    }

    /**
     * @return array
     */
    public function getStoreInWebsiteId()
    {
        $storeIds = [];

        $_list_stores = $this->storeManager->getStores($withDefault = true);
        $_list_websites = $this->storeManager->getWebsites($withDefault = true);

        foreach ($_list_websites as $website) {
            foreach ($_list_stores as $store) {
                if ($website->getId() === $store->getWebsiteId()) {
                    $storeIds[$website->getCode()] = [
                        'store_id' => $store->getId()
                    ];
                    break;
                }
            }
        }

        return $storeIds;
    }

    /**
     * TODO : to be tested
     *
     * @return array
     */
    public function getAllStoresCodeByLocale()
    {
        $stores = $this->storeManager->getStores($withDefault = false);

        //Locale code
        $storesByLocale = [];

        //Try to get list of locale for all stores;
        foreach ($stores as $store) {
            $localeCode = $this->scopeConfig->getValue(
                'general/locale/code',
                ScopeInterface::SCOPE_STORE,
                $store->getStoreId()
            );
            $localeCode = substr($localeCode, 0, 2);
            $storeCode = $store->getCode();
            $storesByLocale[$localeCode][] = $storeCode;
        }

        return $storesByLocale;
    }

    /**
     * @param $locale
     * @return array|mixed
     */
    public function getStoresCodeByLocale($locale)
    {
        if ($locale === '') {
            return [];
        }

        if (count($this->_storesByLocale) === 0) {
            $objectManager = ObjectManager::getInstance();

            $stores = $this->storeManager->getStores($withDefault = false);

            //Locale code
            $this->_storesByLocale = [];

            //Try to get list of locale for all stores;
            foreach ($stores as $store) {
                $localeCode = $this->scopeConfig->getValue(
                    'general/locale/code',
                    ScopeInterface::SCOPE_STORE,
                    $store->getStoreId()
                );
                $localeCode = substr($localeCode, 0, 2);

                if (!isset($this->_storesByLocale[$localeCode])) {
                    $this->_storesByLocale[$localeCode] = [];
                }
                $storeCode = $store->getCode();
                $this->_storesByLocale[$localeCode][] = $storeCode;
            }
        }

        if (isset($this->_storesByLocale[$locale])) {
            return $this->_storesByLocale[$locale];
        }
        return [];
    }

    /**
     * @return mixed
     */
    public function getSkeleton()
    {
        return str_replace(
            ["\n", ' '],
            '',
            '{"field":{"from":"","to":"","type":"text","validators":[],"filters":[],"stores":"","locale":"","actions":[]} }'
        );
    }

    /**
     * @return array
     */
    public function getSpecialActionsPriorities()
    {
        $result = [];

        $storeScope = ScopeInterface::SCOPE_STORE;
        $special_action_config = $this->scopeConfig->getValue(self::CONFIG_SPECIAL_ACTIONS_PRIORITIES, $storeScope);

        if (!$special_action_config) {
            return $result;
        }

        $configData = SerializeHelper::unserialize($special_action_config);

        if (is_object($configData)) {
            $special_action_config = get_object_vars();
        } else {
            $special_action_config = $configData;
        }

        //   $special_action_config = json_decode($special_action_config, true);

        //Reordering
        if (count($special_action_config > 0)) {
            foreach ($special_action_config as $special_action) {
                if (!isset($special_action['name'])) {
                    continue;
                }

                $executed_on = (isset($special_action['executed_on']) ? $special_action['executed_on'] : "");
                $priority = (isset($special_action['priority']) ? $special_action['priority'] : "0");
                $result[$special_action['name']]['executed_on'] = $executed_on;
                $result[$special_action['name']]['priority'] = $priority;
            }
        }

        return $result;
    }

    /**
     * @return array
     */
    public function getListLanguageForAllStores()
    {
        //Locale code
        $allLocale = [];

        $stores = $this->storeManager->getStores($withDefault = false);
        //Try to get list of locale for all stores;
        foreach ($stores as $store) {
            //$locale[] = $this->scopeConfig->getValue('general/locale/code', \Magento\Store\Model\ScopeInterface::SCOPE_STORE, $store->getStoreId());
            $locale = $this->scopeConfig->getValue(
                'general/locale/code',
                ScopeInterface::SCOPE_STORE,
                $store->getStoreId()
            );
            $langCode = explode('_', $locale);
            $lang = trim($langCode[0]);
            if (!isset($allLocale[$lang])) {
                $allLocale[$lang] = [];
            }
            $allLocale[$lang][] = $store->getStoreId();
        }
        return $allLocale;
    }
}
