<?php

namespace FiloBlu\Flow\Model\Actions\From;

use Exception;
use FiloBlu\Flow\Helper\LoggerProvider;
use FiloBlu\Flow\Helper\Product;
use Magento\Catalog\Model\CategoryFactory;
use Magento\Catalog\Model\Product\Type;
use Magento\Catalog\Model\ProductFactory;
use Magento\Catalog\Model\ResourceModel\Category\CollectionFactory;
use Magento\ConfigurableProduct\Model\ResourceModel\Product\Type\Configurable;

/**
 * Class Addtocategoryonlynew
 * @package FiloBlu\Flow\Model\Actions\From
 */
class Addtocategoryonlynew
{

    /**
     * @var @string
     */
    CONST ENTITY_TYPE_CODE = 'catalog_product';

    /**
     * Categories text-path to ID hash.
     *
     * @var array
     */
    protected $_categories = [];
    /**
     * Categories text-path to ID hash with roots checking.
     *
     * @var array
     */
    protected $_logger;
    /**
     * @var array
     */
    protected $_categoriesWithRoots = [];
    /**
     * @var null
     */
    protected $_namesPaths = null;
    /**
     * @var Product
     */
    protected $_helper_product;
    /**
     * @var Configurable
     */
    protected $_configurable_type;
    /**
     * @var CategoryFactory
     */
    protected $_category_factory;
    /**
     * @var ProductFactory
     */
    protected $_product_factory;
    /**
     * @var CollectionFactory
     */
    protected $categoryCollectionFactory;

    /**
     * Addtocategoryonlynew constructor.
     * @param Product $helper_product
     * @param LoggerProvider $loggerProvider
     * @param CategoryFactory $categoryFactory
     * @param ProductFactory $productFactory
     * @param Configurable $configurableProduct
     * @param CollectionFactory $categoryCollectionFactory
     */
    public function __construct(
        Product $helper_product,
        LoggerProvider $loggerProvider,
        CategoryFactory $categoryFactory,
        ProductFactory $productFactory,
        Configurable $configurableProduct,
        CollectionFactory $categoryCollectionFactory

    )
    {
        $this->_logger = $loggerProvider->getLogger();
        $this->_helper_product = $helper_product;
        $this->_category_factory = $categoryFactory;
        $this->_product_factory = $productFactory;
        $this->_configurable_type = $configurableProduct;
        $this->categoryCollectionFactory = $categoryCollectionFactory;
    }

    /**
     * @param $product
     * @param $value
     * @param $storeId
     * @throws Exception
     */
    public function process($product, $value, $storeId)
    {

        if (empty($value)) {
            throw new Exception("Invalid value supplied '{$value}'");
        }

        $old_categories = $product->getCategoryIds();

        if (count($old_categories) > 0) {
            return;
        }

        if (empty($this->_categories)) {
            $this->_initCategories();
        }

        $category = null;
        $originalValue = $value;

        $_category_tree = [];

        // controllo se si tratta di più categorie con separatore |
        $_new_cat_ids = '';
        if (strrpos($originalValue, '|')) {
            $_new_cat_ids = explode('|', $originalValue);
        }
        // gestione posizione
        /*
        $values = explode('#', $value);
        if (count($values) == 2) {
            $value = $values[0];
            $position = $values[1];
        }
        else {
            $value = $values[0];
        }
        */

        // position must be set even if product already exists (EG: positions update)

        if (is_array($_new_cat_ids) && count($_new_cat_ids) > 1) {
            foreach ($_new_cat_ids as $_cat_id) {
                if (is_numeric($_cat_id)) {
                    $category = $this->_category_factory->create()->load($_cat_id);
                    if (!is_object($category) || $category->getId() <= 0) {
                        $this->_logger->info("Category ID {$_cat_id} does not exist - SKIPPED ");
                    } else {
                        $_category_tree = array_merge(explode('/', $category->getPath()), $_category_tree);
                    }
                }
            }
        } else {

            if (is_numeric($value)) {
                $category = $this->_category_factory->create()->load($value);
            } else {
                $category = $this->categoryCollectionFactory
                    ->create()
                    ->addAttributeToFilter('path', array('like' => $this->getCategoryIdsPathByNamesPath($value, false, true)))
                    ->getFirstItem();
                if (is_object($category) && $category->getId() > 0) {
                    $category = $category->load($category->getId());
                }
                if (is_numeric($catId = $this->getIdFromPath($value))) {
                    $category = $this->_category_factory->create()->load($catId);
                }
            }

            if (!is_object($category) || $category->getId() <= 0) {
                throw new Exception("Category ID '{$value}' does not exist");
            }
            $path = $category->getPath();
            $_category_tree = explode('/', $path);
        }

        // mettere in configurazione se le categorie vanno in sostituzione o in update
        // di default in sostituzione

        $_category_update = false;
        $old_categories = $product->getCategoryIds();
        if ($_category_update && count($old_categories)) {
            $merged_category = array_merge($old_categories, $_category_tree);
        } else {
            $merged_category = $_category_tree;
        }

        $merged_category = array_unique($merged_category);
        $merged_category = array_diff($merged_category, [1, 2]); // tolgo root category ID

        //mettere in configurazione se salvare le categorie solo sui configurabili o anche sui semplici
        $_only_configurable_category = $this->_helper_product->getOnlyConfigurableCategoryFlag();
        if ($_only_configurable_category) {

            if ((int)$product->getData('visibility') !== 1) {
                $product->setCategoryIds($merged_category);
                $product->save();
            } else {
                $product->setCategoryIds([]);
                $product->save();
            }
        } else {
            $product->setCategoryIds($merged_category);
            $product->save();
        }


        if ($product->getTypeId() === Type::DEFAULT_TYPE) {
            $parentIds = $this->_configurable_type->getParentIdsByChild($product->getId());
            if (count($parentIds) <= 0) {
                return;
            }

            foreach ($parentIds as $parentId) {
                $parent_product = $this->_product_factory->create()->load($parentId);
                // Sostituzione delle categorie
                // $merged_parent_category = array_unique(array_merge($merged_category,$parent_product->getCategoryIds()));
                $parent_product->setCategoryIds($merged_category)->save();
            }

        }
    }

    /**
     * @return $this
     */
    protected function _initCategories()
    {
        $collection = $this->categoryCollectionFactory->create();
        $collection->setStore(0);
        $collection->addNameToResult();

        foreach ($collection as $category) {
            $structure = explode('/', $category->getPath());
            $pathSize = count($structure);
            if ($pathSize > 1) {
                $path = [];
                for ($i = 1; $i < $pathSize; $i++) {
                    $path[] = $collection->getItemById($structure[$i])->getName();
                }
                $rootCategoryName = array_shift($path);
                if (!isset($this->_categoriesWithRoots[$rootCategoryName])) {
                    $this->_categoriesWithRoots[$rootCategoryName] = [];
                }
                $index = implode('/', $path);
                $index = strtoupper($index);
                $this->_categoriesWithRoots[$rootCategoryName][$index] = $category->getId();
                if ($pathSize > 2) {
                    $this->_categories[$index] = $category->getId();
                }
            }
        }
        return $this;
    }

    /**
     * @param $names_path
     * @param bool $includeRoot
     * @param bool $addLike
     * @return bool|string
     */
    protected function getCategoryIdsPathByNamesPath($names_path, $includeRoot = true, $addLike = false)
    {

        $paths = $this->getCategoryNamesPaths();
        if (!isset($paths['name2id']) || count($paths['name2id']) <= 0) {
            return false;
        }

        $name2id = $paths['name2id'];

        $pathParts = explode('/', $names_path);
        if (!is_array($pathParts) || count($pathParts) <= 0) {
            return false;
        }

        $ids_path_result = [];
        foreach ($pathParts as $singlePathName) {
            if (!isset($name2id[strtolower(trim($singlePathName))])) {
                return false;
            }

            $ids_path_result[] = $name2id[strtolower(trim($singlePathName))];
        }

        $ids_path = implode('/', $ids_path_result);

        if ($addLike) {
            $ids_path = '%' . $ids_path;
        }

        return $ids_path;
    }

    /**
     * @return array|null
     */
    protected function getCategoryNamesPaths()
    {
        if ($this->_namesPaths !== null) {
            return $this->_namesPaths;
        }

        $this->_namesPaths = ['id2name' => [], 'name2id' => []];
        $categoriesCollection = $this->categoryCollectionFactory->create()
            ->setStore(0)
            ->addAttributeToSelect('*');

        foreach ($categoriesCollection as $singleCategory) {
            $categoryName = strtolower(trim($singleCategory->getName()));

            $this->_namesPaths['id2name'][$singleCategory->getId()] = $categoryName;
            $this->_namesPaths['name2id'][$categoryName] = $singleCategory->getId();
        }

        return $this->_namesPaths;

    }

    /**
     * Finds a subcategory id from a path string
     *
     * @param $string
     * @return bool
     */
    public function getIdFromPath($string)
    {
        if (array_key_exists(strtoupper($string), $this->_categories)) {
            return $this->_categories[strtoupper($string)];
        }
        return false;
    }

    /**
     * Returns all valid path strings
     *
     * @return array
     */
    public function getAllPaths()
    {
        return array_keys($this->_categories);
    }

}