<?php

namespace FiloBlu\Flow\Model\From;

use Exception;
use FiloBlu\Flow\Helper\Data;
use FiloBlu\Flow\Model\From\Entity\Bulk;
use FiloBlu\Flow\Model\Inboundflow;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ProductMetadataInterface;
use Magento\Framework\Data\Collection\AbstractDb;
use Magento\Framework\Model\Context;
use Magento\Framework\Model\ResourceModel\AbstractResource;
use Magento\Framework\ObjectManagerInterface;
use Magento\Framework\Registry;

/**
 * Class Eavs
 * @package FiloBlu\Flow\Model\From
 */
class Eavs extends AbstractFrom
{

    /**
     * @var int
     */
    const PRODUCT_NOT_FOUND_ERR_CODE = 1;
    /**
     * @var int
     */
    const SKU_ATTRIBUTE_ERR_CODE = 2;
    /**
     * @var int
     */
    const DROPDOWN_STORE_ERR_CODE = 3;
    /**
     * @var string
     */
    const ENTITY_MANAGER_CLASS_PREFIX = 'FiloBlu\Flow\Model\From\Entity\\';
    /**
     * @var string
     */
    const DEFAULT_ENTITY_TYPE_CODE = 'catalog_product';
    /**
     * @var string
     */
    const ATTRIBUTE_ENTITY_TYPE_SUFFIX = 'attribute';
    /**
     * @var string
     */
    const PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE = 'product_attribute';
    /**
     * @var string
     */
    const EMAIL_FROM = 'Eavs processor <magenteam@.it>';
    /**
     * @var string
     */
    const EMAIL_TO = '';
    /**
     * @var integer
     */
    const PRODUCT_REF_ID = 0;
    /**
     * @var integer
     */
    const ATTRIBUTE_REF_ID = 1;
    /**
     * @var integer
     */
    const SPECIAL_REF_ID = 2;
    /**
     * @var string
     */
    const CONFIG_FLOW_BULKMODE = 'filoblu_flow/flow_process/flow_update_bulkmode';
    /**
     * @var string
     */
    const CONFIG_FLOW_BULKTHRESHOLD = 'filoblu_flow/flow_process/flow_update_bulkmodethreshold';
    /**
     * @var string
     */
    const EXECUTED_ON_ATTRIBUTE = 'attribute';

    /**
     * @var string
     */
    const EXECUTED_ON_PRODUCT = 'product';
    /**
     * @var bool
     */
    protected $_dontRetryErrorCodes = true;
    /**
     * @var int
     */
    protected $_productEntityTypeId = 4;
    /**
     * @var array
     */
    protected $_excludeFromBulk = ['sku'];
    /**
     * @var Entity\Bulk|null
     */
    protected $_bulkModel;
    /**
     * @var ScopeConfigInterface|null
     */
    protected $_scopeConfig;
    /**
     * @var ProductMetadataInterface
     */
    protected $_productMetadata;
    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;
    /**
     * @var Data
     */
    protected $helper_flow;

    /**
     * Eavs constructor.
     * @param ScopeConfigInterface $scopeConfig
     * @param Context $context
     * @param Registry $registry
     * @param Data $helperFlow
     * @param Entity\Bulk $bulkModel
     * @param ObjectManagerInterface $objectManager
     * @param ProductMetadataInterface $productMetadata
     * @param AbstractResource|null $resource
     * @param AbstractDb|null $resourceCollection
     * @param array $data
     */
    public function __construct(
        ScopeConfigInterface $scopeConfig,
        Context $context,
        Registry $registry,
        Data $helperFlow,
        Bulk $bulkModel,
        ObjectManagerInterface $objectManager,
        ProductMetadataInterface $productMetadata,
        AbstractResource $resource = null,
        AbstractDb $resourceCollection = null,

        array $data = []
    ) {
        $this->helper_flow = $helperFlow;
        $this->_bulkModel = $bulkModel;
        $this->_scopeConfig = $scopeConfig;
        $this->_productMetadata = $productMetadata;
        $this->objectManager = $objectManager;
        parent::__construct($context, $registry, $resource, $resourceCollection, $data);
    }

    /**
     * @return void
     */
    public function _construct()
    {
        $this->_init(\FiloBlu\Flow\Model\ResourceModel\From\Eavs::class, 'meta_id');
    }

    /**
     * @return $this|bool|mixed
     * @throws Exception
     */
    public function process()
    {

        // $this->_logger->info(__METHOD__);

        /* check store view
        Only if bulk mode is disabled
        */
        $storesId = $this->getStoreId();

        if ($storesId === false) {
            return $this->error(false, "Store view " . $this->getS() . " is missing");
        }

        $entityTypeCode = $this->getMetaRefId();
        $executedOn = trim($this->getExecutedOn());

        // Default EAVS action, on product!
        if ($entityTypeCode == 0 && strlen($executedOn) < 1) {
            $entityTypeCode = self::DEFAULT_ENTITY_TYPE_CODE;
        // Special action on product
        } elseif ($entityTypeCode > 0 && $executedOn == self::EXECUTED_ON_PRODUCT) {
            $entityTypeCode = self::DEFAULT_ENTITY_TYPE_CODE;
        // Special action on attribute
        } elseif ($entityTypeCode > 0 && $executedOn == self::EXECUTED_ON_ATTRIBUTE) {
            $entityTypeCode = self::PRODUCT_ATTRIBUTE_ENTITY_TYPE_CODE;
        } else {
            $messagee = "Unknown special action type of execution or unknown EAVS action type (e: {$this->getE()} | a: {$this->getA()} | a: {$this->getV()} | meta_ref_id: {$entityTypeCode} | executed_on: {$executedOn})";
            $this->_logger->info($messagee);
            return $this->error(false, $messagee);
        }

        $entityManagerClassName = $this->getEntityManagerClassName($entityTypeCode);

        try {
            foreach ($storesId as $storeId) {
                $entityManager = $this->objectManager->create(
                    $entityManagerClassName,
                    ['data' => [
                        'entityIdentifier' => $this->getE(),
                        'attributeCode' => $this->getA(),
                        'attributeValue' => stripcslashes($this->getV()),
                        'entityTypeCode' => $entityTypeCode,
                        'storeId' => $storeId
                    ]
                    ]
                );
                $entityManager->process();
            }
        } catch (Exception $e) {
            $this->_logger->info($e->getMessage());
            return $this->error(false, $e->getMessage());
        }

        return $this;
    }

    /**
     * @return array|bool
     */
    protected function getStoreId()
    {
        $storeView = $this->getS();
        $language = $this->getL();
        $allStore = $this->helper_flow->getAllStoreIds();
        /* storeview is in s tag with codename */
        if (!empty($storeView)) {
            if (strtolower(trim($storeView)) === 'default') {
                $stores[] = 0;
                return $stores;
            }
            if (isset($allStore[$storeView])) {
                $stores[] = $allStore[$storeView];
                return $stores;
            }
        } elseif (!empty($language)) {
            $languages = $this->helper_flow->getListLanguageForAllStores();
            if (isset($languages[strtolower($language)])) {
                return $languages[strtolower($language)];
            }
        } else {
            $stores[] = 0;
            return $stores;
        }

        return false;
    }

    /**
     * @param $entityTypeCode
     * @return string
     */
    protected function getEntityManagerClassName($entityTypeCode)
    {
        if ((string)$entityTypeCode === self::DEFAULT_ENTITY_TYPE_CODE) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Product';
        } elseif (false !== strpos($entityTypeCode, self::ATTRIBUTE_ENTITY_TYPE_SUFFIX)) {
            return self::ENTITY_MANAGER_CLASS_PREFIX . 'Attribute';
        }

        return self::ENTITY_MANAGER_CLASS_PREFIX . 'AbstractEntity';
    }

    /**
     * @param $file
     * @return bool
     */
    public function sendErrorNotifications($file)
    {
        return true;
    }

    /**
     * @param Inboundflow $inboundFlow
     * @return bool
     */
    protected function postProcess(Inboundflow $inboundFlow)
    {
        parent::postProcess($inboundFlow);

        $bulk_enabled = $this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKMODE);

        if (!$bulk_enabled) {
            return true;
        }

        $this->_logger->info('bulk process : copying attributes from simple products to configurables begins');
        $this->_bulkModel->bulkCopyAttributesToConfigurable($inboundFlow);
        $this->_logger->info('bulk process : complete.');
    }

    /**
     * @param $query
     * @param $file
     * @return mixed
     */
    protected function preprocessFileRows($query, $file)
    {
        $bulk_enabled = $this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKMODE);

        if (!$bulk_enabled) {
            return $query;
        }

        $this->_registry->registry('attribute_configurable_copy');
        $startTime = time();

        try {
            $this->_bulkModel->_insertMissingAttributeOptionValues($file);
            $this->_logger->info('bulk process : missing attribute options inserted in ' . (time() - $startTime) . 's');
        } catch (Exception $e) {
            $this->_logger->info($e->getMessage());
        }

        $startTime = time();

        try {
            $this->_bulkModel->_setMetaStatusToNotFoundProducts($file);
            $this->_logger->info('bulk process : meta status for not found products in ' . (time() - $startTime) . 's');
        } catch (Exception $e) {
            $this->_logger->info($e->getMessage());
        }

        $startTime = time();

        $loop = 0;
        $limit = is_numeric($this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKTHRESHOLD)) ? $this->_scopeConfig->getValue(self::CONFIG_FLOW_BULKTHRESHOLD) : 5000;
        $count = 0;
        $offset = 0;

        while ($rows = $this->_bulkModel->_getMetaAttributesforProducts($file, $limit, $offset)) {
            foreach ($rows as $backend_model => $items) {
                $count += (int)$this->_dumpEAVS($items);
            }

            $loop++;
            $offset = ($limit * $loop) - $count;
        }

        $this->_logger->info('bulk process : values to update got in ' . (time() - $startTime) . 's');

        return $query;
    }

    /**
     * @param $items
     * @return bool
     */
    protected function _dumpEAVS($items)
    {
        if (empty($items)) {
            return false;
        }

        $targetTable = $items[0]['backend_type'];
        $this->_logger->info('Doing ' . sizeof($items) . ' of ' . $items[0]['backend_type']);

        $this->_saveAttributes($items, $targetTable);
        return true;
    }

    /**
     * @param $items
     * @param $targetTable
     * @return bool|int|void
     * @throws Exception
     */
    protected function _saveAttributes($items, $targetTable)
    {
        if (empty($items)) {
            return false;
        }

        if ($targetTable === 'static') {
            throw new Exception(__CLASS__ . ':' . __METHOD__ . ' target table "static" is not supported');
        }

        $Connection = $this->getResource()->getConnection();

        $metaIds = [];
        $toUpdate = [];
        for ($i = 0, $iMax = sizeof($items); $i < $iMax; $i++) {
            $toUpdate[] = sprintf('(%s, %s, %s, %s)', $items[$i]['attribute_id'], $items[$i]['store_id'], $items[$i][$this->getEntityField()], $Connection->quote($items[$i]['v']));
            $metaIds[] = $items[$i]['meta_id'];
        }

        $updateQuery = "INSERT INTO `catalog_product_entity_{$targetTable}`
                            (`attribute_id`, `store_id`, `{$this->getEntityField()}`, `value`)
                        VALUES ";
        $updateQuery .= implode(', ', $toUpdate);
        $updateQuery .= ' ON DUPLICATE KEY UPDATE `value` = VALUES(`value`);';

        $metaUpdateQuery = "UPDATE `flow_from_eavs` SET `meta_processed` = 1 WHERE `meta_id` IN (" . implode(', ', $metaIds) . ");";

        try {
            $Connection->beginTransaction();
            $Connection->query($updateQuery);
            $Connection->query($metaUpdateQuery);
            $Connection->commit();
            return sizeof($items);
        } catch (Exception $e) {
            // $this->lo::log(__METHOD__ . ':' . __LINE__ . ': Rollback happened.' . $e->getMessage());
            $Connection->rollback();
            echo $e->getMessage() . PHP_EOL;
            return false;
        }
    }

    /**
     * @return string
     */
    protected function getEntityField()
    {
        $edition = $this->_productMetadata->getEdition();
        return ($edition === 'Enterprise' || $edition === 'B2B') ? 'row_id' : 'entity_id';
    }
}
