<?php

namespace FiloBlu\Flow\Model\From\Entity;

/**
 * Class AbstractEntity
 * @package FiloBlu\Flow\Model\From\Entity
 */
abstract class AbstractEntity
{
    /**
     * @var mixed|null
     */
    protected $entityTypeCode = null;
    /**
     * @var null
     */
    protected $entityTypeModel = null;
    /**
     * @var mixed|null
     */
    protected $entityIdentifier = null;
    /**
     * @var string|null
     */
    protected $attributeCode = null;
    /**
     * @var string|null
     */
    protected $attributeValue = null;
    /**
     * @var mixed|null
     */
    protected $storeId = null;
    /**
     * @var null
     */
    protected $attributeModel = null;
    /**
     * @var null
     */
    protected $objectEntity = null;
    /**
     * @var null
     */
    protected $_objectEntityId = null;
    /**
     * @var \Magento\Catalog\Model\ProductFactory
     */
    protected $productFactory;
    /**
     * @var \Magento\Framework\Registry
     */
    protected $registry;
    /**
     * @var \FiloBlu\Flow\Helper\Attribute
     */
    protected $helperAttribute;
    /**
     * @var \FiloBlu\Flow\Helper\Monolog\Logger|\Monolog\Logger
     */
    protected $logger;
    /**
     * @var ObjectManagerInterface
     */
    protected $objectManager;

    /**
     * @var \FiloBlu\Flow\Helper\Product
     */
    protected $helperProduct;

    /**
     * @var \Magento\Eav\Model\Entity\AttributeFactory
     */
    protected $attributeFactory;

    /**
     * @var \Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\CollectionFactory
     */
    protected $attributeOptionCollectionFactory;

    /**
     * @var \Magento\Eav\Model\Entity\TypeFactory
     */
    protected $entityTypeFactory;

    /**
     * AbstractEntity constructor.
     * @param \Magento\Framework\Registry $registry
     * @param \Magento\Catalog\Model\ProductFactory $productFactory
     * @param \FiloBlu\Flow\Helper\Product $helperProduct
     * @param \FiloBlu\Flow\Helper\Attribute $helperAttribute
     * @param \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider
     * @param \Magento\Eav\Model\Entity\AttributeFactory $attributeFactory
     * @param \Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\CollectionFactory $attributeOptionCollectionFactory
     * @param \Magento\Framework\ObjectManagerInterface $objectManager
     * @param \Magento\Eav\Model\Entity\TypeFactory $entityTypeFactory
     * @param array $data
     */
    public function __construct(

        \Magento\Framework\Registry $registry,
        \Magento\Catalog\Model\ProductFactory $productFactory,
        \FiloBlu\Flow\Helper\Product $helperProduct,
        \FiloBlu\Flow\Helper\Attribute $helperAttribute,
        \FiloBlu\Flow\Helper\LoggerProvider $loggerProvider,
        \Magento\Eav\Model\Entity\AttributeFactory $attributeFactory,
        \Magento\Eav\Model\ResourceModel\Entity\Attribute\Option\CollectionFactory $attributeOptionCollectionFactory,
        \Magento\Framework\ObjectManagerInterface $objectManager,
        \Magento\Eav\Model\Entity\TypeFactory $entityTypeFactory,
        array $data = []
    )
    {
        $this->attributeOptionCollectionFactory = $attributeOptionCollectionFactory;
        $this->attributeFactory = $attributeFactory;
        $this->helperProduct = $helperProduct;
        $this->objectManager = $objectManager;
        $this->storeId = $data['storeId'];
        $this->attributeCode = trim($data['attributeCode']); // a
        $this->attributeValue = trim($data['attributeValue']); // v
        $this->entityIdentifier = $data['entityIdentifier']; // e
        $this->entityTypeCode = $data['entityTypeCode']; // valori: catalog_product, product_attribute
        $this->productFactory = $productFactory;
        $this->registry = $registry;
        $this->helperAttribute = $helperAttribute;
        $this->entityTypeFactory= $entityTypeFactory;
        $this->logger = $loggerProvider->getLogger();
    }

    /**
     * @return |null
     * @throws \Exception
     */
    protected function getEntityModel()
    {
        if (is_null($this->entityTypeModel)) {
            $entityType = $this->entityTypeFactory->create()->loadByCode($this->getEntityTypeCode());

            if (!$entityType->getId()) {
                throw new \Exception('EntityType not found for code ' . $this->entityTypeCode);
            }

            $this->entityTypeModel = $entityType->getEntityModel();
        }

        return $this->entityTypeModel;
    }

    /**
     * @return mixed|null
     * @throws \Exception
     */
    public function getEntityTypeCode()
    {
        if (empty($this->entityTypeCode)) {
            throw new \Exception('EntityTypeCode was not defined for this class');
        }

        return $this->entityTypeCode;
    }

    /**
     * @throws \Exception
     */
    public function process()
    {
        if ($this->isSpecialFunction($this->attributeCode)) {
            $this->processSpecialFunction();
        } else {
            $this->setObjectAttributeValue();
        }
    }

    /**
     * @param string $attribute
     * @return bool
     */
    public function isSpecialFunction($attribute)
    {
        return (strpos($attribute, '_') === 0);
    }

    /**
     * @return mixed
     * @throws \Exception
     */
    protected function processSpecialFunction()
    {
        $className = $this->getSpecialActionClassName();

        if (class_exists($className)) {
            $specificSpecialClass = $this->objectManager->create($className);

            if (!method_exists($specificSpecialClass, 'process')) {
                throw new \Exception('Missing special function for attribute ' . $this->attributeCode);
            }
            return $specificSpecialClass->process($this->getObject(), $this->attributeValue, $this->storeId);
        }

        throw new \Exception("A special class \"{$className}\" for attribute code (\"{$this->attributeCode}\") was not found");
    }

    /**
     * @return string
     */
    protected function getSpecialActionClassName()
    {
        return '\FiloBlu\Flow\Model\Actions\From\\' . ucfirst(str_replace('_', '', $this->attributeCode));
    }

    /**
     * @return mixed
     */
    abstract protected function getObject();

    /**
     * @throws \Exception
     */
    protected function setObjectAttributeValue()
    {
        $object = $this->getObject();
        $attributeValue = $this->getAttributeValue();
        $input_type = strtolower($this->getAttribute()->getFrontend()->getInputType());

        if ($input_type === 'multiselect' && empty($attributeValue)) {
            $object->unsetData($this->attributeCode);
            $object->save();
        } elseif (is_null($attributeValue) || empty($attributeValue)) {
            $object->unsetData($this->attributeCode);
        } else {
            $object->setDataUsingMethod($this->attributeCode, $attributeValue);
            $object->getResource()->saveAttribute($object, $this->attributeCode);
        }

    }

    /**
     * @return mixed|string|null
     * @throws \Exception
     */
    protected function getAttributeValue()
    {
        $attribute_model = $this->getAttribute();
        $attributeValue = $this->attributeValue;
        $input_type = strtolower($attribute_model->getFrontend()->getInputType());

        if (strlen($attributeValue) > 0 && ($input_type === 'select' || $input_type === 'multiselect') && $this->getAttributeCode() != 'visibility' && $this->getAttributeCode() != 'country_of_manufacture') {
            $attributeValueLabels = [];
            $attributeValueArray = [];
            if ($input_type === 'multiselect') {
                if (strpos($attributeValue, '|') !== false) {
                    //if there is | as separator, we need to replace with ,
                    $attributeValue = str_replace('|', ',', $attributeValue);
                }
                $attributeValueLabels = explode(',', $attributeValue);
                $attributeValueLabels = array_map('trim', $attributeValueLabels);
                $attributeValueLabels = array_unique($attributeValueLabels);
            } else {
                $attributeValueLabels[] = $attributeValue;
            }
            foreach ($attributeValueLabels as $attributeValueLabel) {
                $optionId = $this->getAttributeOptionValue($attributeValueLabel);
                if (empty($optionId) && $this->storeId == 0) { // && Mage::getStoreConfig('fbflow/new_options_status') == 1) {
                    $this->addAttributeOptionValue($attributeValueLabel);
                    $optionId = $this->getAttributeOptionValue($attributeValueLabel);
                }
                if (empty($optionId)) {
                    throw new \Exception('Couldn\'t find nor add option ' . $attributeValueLabel . ' for attribute ' . $this->attributeCode . ' entity type ' . $this->getEntityTypeCode() . ' on store ' . $this->storeId);
                }
                $attributeValueArray[] = $optionId;
            }
            $attributeValue = implode(',', $attributeValueArray);
        }

        return $attributeValue;
    }

    /**
     * @return |null
     * @throws \Exception
     */
    protected function getAttribute()
    {
        if (empty($this->attributeModel)) {
            $this->attributeModel = $this->attributeFactory->create()->loadByCode($this->getEntityTypeCode(), $this->getAttributeCode());
            if (!$this->attributeModel->getId()) {
                throw new \Exception("No attribute found for code {$this->attributeCode}");
            }
        }
        return $this->attributeModel;
    }

    /**
     * @return string|null
     * @throws \Exception
     */
    public function getAttributeCode()
    {
        if (empty($this->attributeCode)) {
            throw new \Exception('AttributeCode was not defined for this class');
        }

        return $this->attributeCode;
    }

    /**
     * @param $attributeValueLabel
     * @return mixed
     */
    protected function getAttributeOptionValue($attributeValueLabel)
    {

        $collection = $this->attributeOptionCollectionFactory->create();

        $collection->getSelect()
            ->join('eav_attribute', 'eav_attribute.attribute_id=main_table.attribute_id', 'eav_attribute.attribute_code')
            ->join('eav_attribute_option_value', 'eav_attribute_option_value.option_id=main_table.option_id', 'value');

        $collection
            ->addFieldToFilter('eav_attribute_option_value.value', ['eq' => $attributeValueLabel])
            ->addFieldToFilter('attribute_code', ['eq' => $this->attributeCode])
            ->addFieldToFilter('store_id', ['eq' => $this->storeId]);

        $attribute = $collection->getFirstItem();

        return $attribute->getOptionId();
    }

    /**
     * @param $attributeValueLabel
     * @return bool
     * @throws \Exception
     */
    protected function addAttributeOptionValue($attributeValueLabel)
    {
        try {

            return $this->helperAttribute->addAttributeOption(
                [
                    'attribute_id' => $this->getAttribute()->getId(),
                    'store_id' => 0,
                    'value' => $attributeValueLabel
                ]
            );

        } catch (\Exception $e) {
            throw new \Exception('Couldn\'t add option ' . $attributeValueLabel . ' for attribute ' . $this->attributeCode . ' entity type ' . $this->getEntityTypeCode());
        }

    }

    /**
     * @return mixed|null
     */
    public function getObjectId()
    {
        if (is_null($this->_objectEntityId)) {
            $this->_objectEntityId = $this->entityIdentifier;
        }
        return $this->_objectEntityId;
    }

}
